# Copyright (C) 2023 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: MIT

from __future__ import annotations

# Import this module for its side effects
import _libdnf5_shim.initialize  # noqa: F401
